package com.jaywagnon.mymvc.configuration;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tomcat.dbcp.dbcp.BasicDataSource;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.dao.annotation.PersistenceExceptionTranslationPostProcessor;
import org.springframework.orm.hibernate4.HibernateTransactionManager;
import org.springframework.orm.hibernate4.LocalSessionFactoryBean;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.sql.DataSource;
import java.util.Properties;

import static com.jaywagnon.mymvc.configuration.ConfigurationEnum.*;


@Configuration
@EnableTransactionManagement
public class DataSourceConfiguration
{
  private Log log = LogFactory.getLog(this.getClass());


  /**
   * Returns a DataSource object instantiated using configured
   * settings for:
   * <p/>
   * DATA_SOURCE_URL
   * DATA_SOURCE_DRIVER
   * DATA_SOURCE_USER
   * DATA_SOURCE_PASSWORD
   *
   * @return
   */
  @Bean(destroyMethod = "close")
  public DataSource dataSource()
  {
    BasicDataSource dataSource = new BasicDataSource();

    String url = DATA_SOURCE_URL.value();
    String driver = DATA_SOURCE_DRIVER.value();

    dataSource.setUrl(url);
    dataSource.setDriverClassName(driver);
    dataSource.setUsername(DATA_SOURCE_USER.value());
    dataSource.setPassword(DATA_SOURCE_PASSWORD.value());

    logDataSource(url, driver);

    return dataSource;
  }


  /**
   * Returns a Properties object with the Hibernate-specific settings.
   *
   * @return
   */
  @Bean
  public Properties getHibernateProperties()
  {
    Properties properties = new Properties();
    properties.put("hibernate.dialect", HIBERNATE_DIALECT.value());
    properties.put("hibernate.hbm2ddl.auto", HIBERNATE_HDM2DDL_AUTO.value());
    properties.put("hibernate.show_sql", HIBERNATE_SHOW_SQL.value());
    return properties;
  }


  /**
   * Returns a session factory for Hibernate.
   *
   * @return
   */
  @Bean
  public LocalSessionFactoryBean sessionFactory()
  {
    LocalSessionFactoryBean sessionFactory = new LocalSessionFactoryBean();
    sessionFactory.setDataSource(dataSource());
    sessionFactory.setHibernateProperties(getHibernateProperties());
    sessionFactory.setPackagesToScan(new String[]{"com.jaywagnon.mymvc.persistence"});
    return sessionFactory;
  }


  /**
   * Returns a transaction manager for Hibernate.
   *
   * @param sessionFactory
   * @return
   */
  @Bean
  @Autowired
  public HibernateTransactionManager transactionManager(SessionFactory sessionFactory)
  {
    HibernateTransactionManager manager = new HibernateTransactionManager();
    manager.setSessionFactory(sessionFactory);
    return manager;
  }


  /**
   * Returns an exception translator for persistence exceptions.
   *
   * @return
   */
  @Bean
  public PersistenceExceptionTranslationPostProcessor exceptionTranslation()
  {
    return new PersistenceExceptionTranslationPostProcessor();
  }


  /**
   * Outputs the DATA_SOURCE_URL and DATA_SOURCE_DRIVER used
   * to instantiate the DataSource object from dataSource().
   *
   * @param url
   * @param driver
   */
  private void logDataSource(String url, String driver)
  {
    StringBuilder builder = new StringBuilder();
    builder.append("Registering DataSource:\n");
    builder.append("\tURL:     ").append(url).append("\n");
    builder.append("\tDriver:  ").append(driver);

    log.debug(builder.toString());
  }
}
